// Class CGUide, Version 4.0, Feb 2008

#include <afxwin.h>
#include <afxsock.h>	// CSocket Class

#include "CGuide.h"
#include "CEngineDef.h"


CDraughtsSocket::CDraughtsSocket()
{
}


CDraughtsSocket::~CDraughtsSocket()
{
}


void CDraughtsSocket::OnAccept( int iErrorCode )
{
	if ( iErrorCode == 0 ) {
		m_bAccept = true ;
		m_pGuide->OnAccept() ;
	}
}


void CDraughtsSocket::OnClose( int iErrorCode )
{
		m_pGuide->OnClose() ;
}


void CDraughtsSocket::OnConnect( int iErrorCode )
{
	if ( iErrorCode == 0 ) {
		m_bConnect = true ;
		m_pGuide->OnConnect() ;
	}
}


void CDraughtsSocket::OnReceive( int iErrorCode )
{
	if ( iErrorCode == 0 )
		m_pGuide->OnReceive() ;
}


CGuide::CGuide()
{
}


CGuide::~CGuide()
{
}


bool CGuide::Init(int iGUIDE_Mode, int iCommunicationMode, int iPort, CString csIPAddress)
{
	BOOL bCreate ;

	// Socket Communication Initialization
	if ( !AfxSocketInit() ) {
		AfxMessageBox( "Socket initialization Error.", MB_ICONEXCLAMATION ) ;
		return false ;
	}

	if ( iGUIDE_Mode == GUIDE_GUI )
		m_iGUIDE_Mode = GUIDE_CLIENT ;
	else if ( iGUIDE_Mode == GUIDE_ENGINE )
		m_iGUIDE_Mode = GUIDE_SERVER ;
	else
		return false ;

	m_iCommunicationMode = iCommunicationMode ;

	if ( iPort != -1 )
		m_Port = iPort ;
	else
		m_Port = SOCKET_DEFAULT_PORT ;

	m_bConnect	= false ;
	m_IP		= csIPAddress ;

	m_ConnectSocket.m_pGuide	= this ;
	m_ListenSocket.m_pGuide		= this ;

	m_iNPacketReceive			= 0 ;
	m_csReceiveBuffer.Empty() ;

	if ( iGUIDE_Mode == GUIDE_SERVER ) {
		bCreate = m_ListenSocket.Create(m_Port) ;
		if ( !bCreate )
			return false ;
		m_ListenSocket.Listen() ;
	} else {
		bCreate = m_ConnectSocket.Create() ; 
		if ( !bCreate )
			return false ;

		InitConnectThread() ;
	}

	m_iIndex_GUIMESSAGE_READ = 0 ;

	return true ;
}


bool CGuide::Close()
{

	return true ;
}


bool CGuide::SendMessageToEngine(CString cGUIMessage )
{
	int iLen ;
	CString	csData ;

//	csData.Format("%s%c", cGUIMessage, GUIDE_MESSAGE_DELIMITER) ;
	csData.Format("%s\n", cGUIMessage) ;
	iLen = csData.GetLength() ;

    m_ConnectSocket.Send(csData, iLen) ;

	return true ;
}


bool CGuide::SendMessageToGUI(CString csEngineMessage) 
{
	int iLen ;
	CString	csData ;

//	csData.Format("%s%c", cEngineMessage, GUIDE_MESSAGE_DELIMITER) ;

	if ( m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_GUIDE ) {
		csData.Format("%s\n", csEngineMessage) ;
	}

	if ( m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_DXP ) {
		csData.Format("%s", csEngineMessage) ;
		csData += '\0' ;
	}

	iLen = csData.GetLength() ;	

    m_ConnectSocket.Send(csData, iLen) ;

	return true ;
}


bool CGuide::OnAccept()
{
	m_ListenSocket.Accept(m_ConnectSocket) ;
	
	m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, GUIDE_NOTIFICATION_ONACCEPT ) ;

	return true ;
}


bool CGuide::OnClose()
{
	
	m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, GUIDE_NOTIFICATION_ONCLOSE ) ;

	return true ;
}


bool CGuide::OnConnect()
{
	
	m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, GUIDE_NOTIFICATION_ONCONNECT ) ;

	return true ;
}


bool CGuide::OnDXPGameAcc()
{
	
	m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, GUIDE_NOTIFICATION_ONDXPGAMEACC ) ;

	return true ;
}


bool CGuide::OnDXPGameEnd()
{
	
	m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, GUIDE_NOTIFICATION_ONDXPGAMEEND ) ;

	return true ;
}


bool CGuide::OnReceive()
{

	char *cBuffer = new char[GUIDE_BUFFER_SIZE + 1] ;
    int iBufferSize = GUIDE_BUFFER_SIZE ; 
    int i, iLength, iStart, iRcvd ; 
	int iIndex ;
	CString csBuffer, csMessage ;

	iRcvd = m_ConnectSocket.Receive( cBuffer, iBufferSize) ;

	cBuffer[iRcvd]	= NULL ; 
	csBuffer		= cBuffer ;

	if ( csBuffer.GetLength() )
		++m_iNPacketReceive ;		// Count Packages with length > 0 
	
// Temporary Implementation
	if ( m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_GUIDE ) {

		iLength = m_csReceiveBuffer.GetLength() ;

		if ( iLength ) {	// Part of Engine Message not completely processed !
			csBuffer = m_csReceiveBuffer + csBuffer ;
			m_csReceiveBuffer.Empty() ;
			iRcvd += iLength ;
		}

		i		= 0 ;
		iStart	= 0 ;
		while ( i < iRcvd ) {

			if ( csBuffer.GetAt(i) == GUIDE_MESSAGE_DELIMITER )	{

				csMessage	= csBuffer.Mid(iStart, i-iStart) ;
				iStart		= i+1 ;
				iLength		= csMessage.GetLength() ;

				if ( iLength > 0 ) {

					iIndex		= m_iIndex_GUIMESSAGE_READ % GUIDE_GUIMESSAGE_READ_QUEUE_SIZE ;

					m_csGUIMessage[iIndex] = csMessage ; 

					m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, iIndex ) ;
			
					++m_iIndex_GUIMESSAGE_READ ;

				}
			}

			i++ ;
		}
	
		if ( cBuffer[iRcvd-1] != GUIDE_MESSAGE_DELIMITER )
			m_csReceiveBuffer = csBuffer.Right(iRcvd - iStart) ;
	}

	if ( m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_DXP ) {
				
		csMessage = csBuffer ;
	
		iIndex = m_iIndex_GUIMESSAGE_READ % GUIDE_GUIMESSAGE_READ_QUEUE_SIZE ;

		m_csGUIMessage[iIndex] = csMessage ; 

		m_pWnd->PostMessage(WM_USER_GUIDE_MESSAGE, iIndex ) ;
			
		++m_iIndex_GUIMESSAGE_READ ;
	}

	return true ;
}


bool CGuide::CommandMessage(CString csMessage, CString* pcsMessageCommand, CString* pcsMessageBody )

//	Return true if MessageCommand is found otherwise false

{
	int iFind, iLength ;

	iLength = csMessage.GetLength() ;
	
	if ( iLength == 0 ) {
		pcsMessageCommand->Empty() ;
		pcsMessageBody->Empty() ;

		return false ;
	}

	csMessage.TrimLeft() ;			// Trim spaces

	iFind = csMessage.Find(' ') ;	// Find first space (End of Message Command )

	if ( iFind == -1 )				// No additional command data
		*pcsMessageCommand = csMessage ;
	else
		*pcsMessageCommand = csMessage.Left(iFind) ;

	if ( iFind ) {
		*pcsMessageBody = csMessage.Right(iLength - iFind) ;
		pcsMessageBody->TrimLeft() ;
	} else
		pcsMessageBody->Empty() ;

	return true ;
}


bool CGuide::ParseMessage(CString* pcsMessage, CString* pcsToken )

//	Return true if Token found, otherwise false

{
	int iFind, iLength ;

	pcsMessage->TrimLeft() ;		// Trim spaces
	pcsToken->Empty() ;				// Empty Token

	iLength = pcsMessage->GetLength() ;

	if ( iLength == 0 )				// No Next Token !
		return false ;

	iFind = pcsMessage->Find(' ') ;	// Find first space (End of Message Command )

	if ( iFind == -1 ) {			// No Next Token !
		*pcsToken = *pcsMessage ;
		pcsMessage->Empty() ;
	} else
		*pcsToken = pcsMessage->Left(iFind) ;

	if ( iFind ) {
		*pcsMessage = pcsMessage->Right(iLength - iFind) ;
		pcsMessage->TrimLeft() ;
	} else
		pcsMessage->Empty() ;

	return true ;
}


void CGuide::InitConnectThread()
{

	m_pConnectThread = AfxBeginThread(ConnectThreadFunc, this) ;

}


UINT CGuide::ConnectThreadFunc(LPVOID me)
{

	CGuide* self = (CGuide*)me ;
	self->runConnect() ;

	return ( 0 ) ;
}


void CGuide::runConnect(void)
{
	BOOL bConnect ;
	
	bConnect = 0 ;

	while ( !bConnect ) {
	
		Sleep(200) ;
		bConnect = m_ConnectSocket.Connect(m_IP, m_Port);
 
	}

	m_bConnect = true ;
}


